@@ -27,7 +27,7 @@
 #define NAUTILUS_ICON_CONTAINER_H
 
 #include <eel/eel-canvas.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
+#include <libnautilus-private/nautilus-icon-info.h>
 
 #define NAUTILUS_ICON_CONTAINER(obj) \
 	GTK_CHECK_CAST (obj, nautilus_icon_container_get_type (), NautilusIconContainer)
@@ -56,6 +56,7 @@ typedef struct {
 
 typedef enum {
 	NAUTILUS_ICON_LAYOUT_L_R_T_B,
+	NAUTILUS_ICON_LAYOUT_R_L_T_B,
 	NAUTILUS_ICON_LAYOUT_T_B_L_R,
 	NAUTILUS_ICON_LAYOUT_T_B_R_L
 } NautilusIconLayoutMode;
@@ -127,10 +128,12 @@ typedef struct {
 	 * These must be implemented. The default "do nothing" is not
 	 * good enough, these are _not_ signals.
 	 */
-	char *       (* get_icon_images)          (NautilusIconContainer *container,
+	NautilusIconInfo *(* get_icon_images)     (NautilusIconContainer *container,
 						   NautilusIconData *data,
-						   GList **emblem_icons,
+						   int icon_size,
+						   GList **emblem_pixbufs,
 						   char **embedded_text,
+						   gboolean for_drag_accept,
 						   gboolean need_large_embeddded_text,
 						   gboolean *embedded_text_needs_loading,
 						   gboolean *has_window_open);
@@ -146,6 +149,8 @@ typedef struct {
 	int          (* compare_icons_by_name)    (NautilusIconContainer *container,
 						   NautilusIconData *icon_a,
 						   NautilusIconData *icon_b);
+	void         (* freeze_updates)           (NautilusIconContainer *container);
+	void         (* unfreeze_updates)         (NautilusIconContainer *container);
 	void         (* start_monitor_top_left)   (NautilusIconContainer *container,
 						   NautilusIconData *data,
 						   gconstpointer client,
@@ -297,4 +302,20 @@ gboolean          nautilus_icon_container_get_allow_moves               (Nautilu
 void              nautilus_icon_container_set_allow_moves               (NautilusIconContainer  *container,
 									 gboolean                allow_moves);
 
+gboolean	  nautilus_icon_container_is_layout_rtl			(NautilusIconContainer  *container);
+
+void              nautilus_icon_container_widget_to_file_operation_position (NautilusIconContainer *container,
+									     GdkPoint              *position);
+
+
+#define CANVAS_WIDTH(container) ((GTK_WIDGET (container)->allocation.width \
+				- container->details->left_margin \
+				- container->details->right_margin) \
+				/  EEL_CANVAS (container)->pixels_per_unit)
+
+#define CANVAS_HEIGHT(container) ((GTK_WIDGET (container)->allocation.height \
+			 - container->details->top_margin \
+			 - container->details->bottom_margin) \
+			 / EEL_CANVAS (container)->pixels_per_unit)
+
 #endif /* NAUTILUS_ICON_CONTAINER_H */
