@@ -150,7 +150,7 @@ nautilus_icon_info_new_for_icon_info (GtkIconInfo *icon_info)
 
 	icon->display_name = g_strdup (gtk_icon_info_get_display_name (icon_info));
 
-	filename = gtk_icon_info_get_display_name (icon_info);
+	filename = gtk_icon_info_get_filename (icon_info);
 	if (filename != NULL) {
 		basename = g_path_get_basename (filename);
 		p = strrchr (basename, '.');
@@ -353,11 +353,25 @@ nautilus_icon_info_lookup (GIcon *icon,
 					       size,
 					       NULL, NULL, NULL);
 		if (stream) {
+			GdkPixbuf *scaled_pixbuf;
+		        int w, h, s;
+			double scale;
+			
 			pixbuf = eel_gdk_pixbuf_load_from_stream (stream);
-
-			/* TODO: resize icon? */
-
 			g_object_unref (stream);
+			
+			w = gdk_pixbuf_get_width (pixbuf);
+			h = gdk_pixbuf_get_height (pixbuf);
+			s = MAX (w, h);
+
+			if (size != s) {
+				scale = (double)size / s;
+				scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
+									 w * scale, h * scale,
+									 GDK_INTERP_HYPER);
+				g_object_unref (pixbuf);
+				pixbuf = scaled_pixbuf;
+			}
 		}
 
 		icon_info = nautilus_icon_info_new_for_pixbuf (pixbuf);
@@ -437,11 +451,13 @@ nautilus_icon_info_get_pixbuf_nodefault (NautilusIconInfo  *icon)
 		res = NULL;
 	} else {
 		res = g_object_ref (icon->pixbuf);
-		icon->sole_owner = FALSE;
-		
-		g_object_add_toggle_ref (G_OBJECT (res),
-					 pixbuf_toggle_notify,
-					 icon);
+
+		if (icon->sole_owner) {
+			icon->sole_owner = FALSE;
+			g_object_add_toggle_ref (G_OBJECT (res),
+						 pixbuf_toggle_notify,
+						 icon);
+		}
 	}
 	
 	return res;
@@ -485,6 +501,10 @@ nautilus_icon_info_get_pixbuf_nodefault_at_size (NautilusIconInfo  *icon,
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
 	s = MAX (w, h);
+	if (s == forced_size) {
+		return pixbuf;
+	}
+
 	scale = (double)forced_size / s;
 	scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
 						 w * scale, h * scale,
@@ -507,6 +527,10 @@ nautilus_icon_info_get_pixbuf_at_size (NautilusIconInfo  *icon,
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
 	s = MAX (w, h);
+	if (s == forced_size) {
+		return pixbuf;
+	}
+
 	scale = (double)forced_size / s;
 	scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
 						 w * scale, h * scale,
