@@ -34,12 +34,12 @@
 
 #include "fm-tree-model.h"
 #include "fm-properties-window.h"
+#include <string.h>
+#include <eel/eel-alert-dialog.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-preferences.h>
-#include <eel/eel-string.h>
 #include <eel/eel-stock-dialogs.h>
-#include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcellrenderertext.h>
@@ -56,17 +56,19 @@
 #include <gtk/gtkmenushell.h>
 #include <gtk/gtkclipboard.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomeui/gnome-popup-menu.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libnautilus-private/nautilus-clipboard-monitor.h>
+#include <libnautilus-private/nautilus-desktop-icon-file.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
+#include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-program-choosing.h>
 #include <libnautilus-private/nautilus-tree-view-drag-dest.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-cell-renderer-pixbuf-emblem.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
@@ -86,6 +88,8 @@ struct FMTreeViewDetails {
 	GtkTreeModelSort *sort_model;
 	FMTreeModel *child_model;
 
+	GVolumeMonitor *volume_monitor;
+
 	NautilusFile *activation_file;
 	gboolean activation_in_new_window;
 
@@ -105,6 +109,7 @@ struct FMTreeViewDetails {
 	GtkWidget *popup_paste;
 	GtkWidget *popup_rename;
 	GtkWidget *popup_trash;
+	GtkWidget *popup_delete;
 	GtkWidget *popup_properties;
 	GtkWidget *popup_unmount_separator;
 	GtkWidget *popup_unmount;
@@ -119,6 +124,7 @@ typedef struct {
 } PrependURIParameters;
 
 static GdkAtom copied_files_atom;
+static gboolean show_delete_command_auto_value;
 
 enum {
 	GNOME_COPIED_FILES
@@ -208,7 +214,7 @@ show_selection_idle_callback (gpointer callback_data)
 
 	view->details->show_selection_idle_id = 0;
 
-	file = nautilus_file_get (view->details->selection_location);
+	file = nautilus_file_get_by_uri (view->details->selection_location);
 	if (file == NULL) {
 		return FALSE;
 	}
@@ -285,7 +291,7 @@ row_loaded_callback (GtkTreeModel     *tree_model,
 	}
 
 	/* if iter is ancestor of wanted selection_location then update selection */
-	selection_file = nautilus_file_get (view->details->selection_location);
+	selection_file = nautilus_file_get_by_uri (view->details->selection_location);
 	while (selection_file != NULL) {
 		if (file == selection_file) {
 			nautilus_file_unref (file);
@@ -328,6 +334,7 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
         FMTreeView *view;
 	GdkScreen *screen;
 	NautilusWindowOpenMode mode;
+	GFile *location;
 	
         view = FM_TREE_VIEW (callback_data);
 
@@ -337,38 +344,38 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
         
         mode = view->details->activation_in_new_window ? NAUTILUS_WINDOW_OPEN_IN_NAVIGATION : NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE;
 
-	/* FIXME: reenable && !eel_uris_match_ignore_fragments (view->details->current_main_view_uri, uri) */
-
 	uri = nautilus_file_get_activation_uri (file);
-	if (uri != NULL
-	    && eel_str_has_prefix (uri, NAUTILUS_COMMAND_SPECIFIER)) {
-
-		uri += strlen (NAUTILUS_COMMAND_SPECIFIER);
-		nautilus_launch_application_from_command (screen, NULL, uri, NULL, FALSE);
-
-	} else if (uri != NULL
-	    	   && eel_str_has_prefix (uri, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
-		   
+	if (nautilus_file_is_launcher (file)) {
 		file_uri = nautilus_file_get_uri (file);
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "tree view launch_desktop_file window=%p: %s",
+				    view->details->window, file_uri);
 		nautilus_launch_desktop_file (screen, file_uri, NULL, NULL);
 		g_free (file_uri);
-		
 	} else if (uri != NULL
 		   && nautilus_file_is_executable (file)
 		   && nautilus_file_can_execute (file)
 		   && !nautilus_file_is_directory (file)) {	
 		   
-		file_uri = gnome_vfs_get_local_path_from_uri (uri);
+		file_uri = g_filename_from_uri (uri, NULL, NULL);
 
 		/* Non-local executables don't get launched. They act like non-executables. */
 		if (file_uri == NULL) {
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "tree view window_info_open_location window=%p: %s",
+					    view->details->window, uri);
+			location = g_file_new_for_uri (uri);
 			nautilus_window_info_open_location
 				(view->details->window, 
-				 uri, 
+				 location, 
 				 mode,
 				 0,
 				 NULL);
+			g_object_unref (location);
 		} else {
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "tree view launch_application_from_command window=%p: %s",
+					    view->details->window, file_uri);
 			nautilus_launch_application_from_command (screen, NULL, file_uri, NULL, FALSE);
 			g_free (file_uri);
 		}
@@ -380,12 +387,17 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 				g_free (view->details->selection_location);
 			}
 			view->details->selection_location = g_strdup (uri);
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "tree view window_info_open_location window=%p: %s",
+					    view->details->window, uri);
+			location = g_file_new_for_uri (uri);
 			nautilus_window_info_open_location
 				(view->details->window, 
-				 uri,
+				 location,
 				 mode,
 				 0,
 				 NULL);
+			g_object_unref (location);
 		}
 	}
 
@@ -446,7 +458,7 @@ selection_changed_timer_callback(FMTreeView *view)
 	}
 	view->details->activation_in_new_window = FALSE;
 		
-	attributes = NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI;
+	attributes = NAUTILUS_FILE_ATTRIBUTE_INFO | NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	nautilus_file_call_when_ready (view->details->activation_file, attributes,
 				       got_activation_uri_callback, view);
 	return FALSE; /* remove timeout */
@@ -495,9 +507,9 @@ compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer call
 	}
 
 	/* don't sort root nodes */
-	if (fm_tree_model_iter_is_root (FM_TREE_MODEL (model), a)
-	    || fm_tree_model_iter_is_root (FM_TREE_MODEL (model), b)) {
-		return 0;
+	if (fm_tree_model_iter_is_root (FM_TREE_MODEL (model), a) &&
+	    fm_tree_model_iter_is_root (FM_TREE_MODEL (model), b)) {
+		return fm_tree_model_iter_compare_roots (FM_TREE_MODEL (model), a, b);
 	}
 
 	file_a = fm_tree_model_iter_get_file (FM_TREE_MODEL (model), a);
@@ -569,57 +581,49 @@ move_copy_items_callback (NautilusTreeViewDragDest *dest,
 }
 
 static void
-theme_changed_callback (GObject *icon_factory, gpointer callback_data)
+add_root_for_mount (FMTreeView *view,
+		     GMount *mount)
 {
-        FMTreeView *view; 
+	char *mount_uri, *name;
+	GFile *root;
+	GIcon *icon;
 
-        view = FM_TREE_VIEW (callback_data);
-	if (view->details->child_model != NULL) {
-		fm_tree_model_set_theme (FM_TREE_MODEL (view->details->child_model));
-	}
-}
-
-static void
-add_root_for_volume (FMTreeView *view,
-		     GnomeVFSVolume *volume)
-{
-	char *icon, *mount_uri, *name;
-
-	if (!gnome_vfs_volume_is_user_visible (volume)) {
-		return;
-	}
-	
-	icon = gnome_vfs_volume_get_icon (volume);
-	mount_uri = gnome_vfs_volume_get_activation_uri (volume);
-	name = gnome_vfs_volume_get_display_name (volume);
+	icon = g_mount_get_icon (mount);
+	root = g_mount_get_root (mount);
+	mount_uri = g_file_get_uri (root);
+	g_object_unref (root);
+	name = g_mount_get_name (mount);
 	
 	fm_tree_model_add_root_uri(view->details->child_model,
-				   mount_uri, name, icon, volume);
+				   mount_uri, name, icon, mount);
 
-	g_free (icon);
+	g_object_unref (icon);
 	g_free (name);
 	g_free (mount_uri);
 	
 }
 
 static void
-volume_mounted_callback (GnomeVFSVolumeMonitor *volume_monitor,
-			 GnomeVFSVolume *volume,
-			 FMTreeView *view)
+mount_added_callback (GVolumeMonitor *volume_monitor,
+		      GMount *mount,
+		      FMTreeView *view)
 {
-	add_root_for_volume (view, volume);
+	add_root_for_mount (view, mount);
 }
 
 static void
-volume_unmounted_callback (GnomeVFSVolumeMonitor *volume_monitor,
-			   GnomeVFSVolume *volume,
-			   FMTreeView *view)
+mount_removed_callback (GVolumeMonitor *volume_monitor,
+			GMount *mount,
+			FMTreeView *view)
 {
+	GFile *root;
 	char *mount_uri;
-	
-	mount_uri = gnome_vfs_volume_get_activation_uri (volume);
+
+	root = g_mount_get_root (mount);
+	mount_uri = g_file_get_uri (root);
+	g_object_unref (root);
 	fm_tree_model_remove_root_uri (view->details->child_model,
-					     mount_uri);
+				       mount_uri);
 	g_free (mount_uri);
 }
 
@@ -646,56 +650,22 @@ get_clipboard (GtkWidget *widget)
 }
 
 static gboolean
-can_move_uri_to_trash (const char *file_uri_string)
+is_parent_writable (NautilusFile *file)
 {
-	/* Return TRUE if we can get a trash directory on the same volume as this file. */
-	GnomeVFSURI *file_uri;
-	GnomeVFSURI *directory_uri;
-	GnomeVFSURI *trash_dir_uri;
+	NautilusFile *parent;
 	gboolean result;
-
-	g_return_val_if_fail (file_uri_string != NULL, FALSE);
-
-	file_uri = gnome_vfs_uri_new (file_uri_string);
-
-	if (file_uri == NULL) {
-		return FALSE;
-	}
-
-	/* FIXME: Why can't we just pass file_uri to gnome_vfs_find_directory? */
-	directory_uri = gnome_vfs_uri_get_parent (file_uri);
-	gnome_vfs_uri_unref (file_uri);
-
-	if (directory_uri == NULL) {
-		return FALSE;
-	}
-
-	/*
-	 * Create a new trash if needed but don't go looking for an old Trash.
-	 * Passing 0 permissions as gnome-vfs would override the permissions 
-	 * passed with 700 while creating .Trash directory
-	 */
-	result = gnome_vfs_find_directory (directory_uri, GNOME_VFS_DIRECTORY_KIND_TRASH,
-					   &trash_dir_uri, TRUE, FALSE, 0) == GNOME_VFS_OK;
-	if (result) {
-		gnome_vfs_uri_unref (trash_dir_uri);
-	}
-	gnome_vfs_uri_unref (directory_uri);
-
-	return result;
-}
-
-static gboolean
-eject_for_type (GnomeVFSDeviceType type)
-{
-	switch (type) {
-	case GNOME_VFS_DEVICE_TYPE_CDROM:
-	case GNOME_VFS_DEVICE_TYPE_ZIP:
-	case GNOME_VFS_DEVICE_TYPE_JAZ:
-		return TRUE;
-	default:
+	
+	parent = nautilus_file_get_parent (file);
+	
+	/* No parent directory, return FALSE */
+	if (parent == NULL) {
 		return FALSE;
 	}
+	
+	result = nautilus_file_can_write (parent);
+	nautilus_file_unref (parent);
+	
+	return result;	
 }
 
 static gboolean
@@ -703,12 +673,16 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 			 FMTreeView *view)
 {
 	GtkTreePath *path, *cursor_path;
-	char *uri;
+	gboolean parent_file_is_writable;
+	gboolean file_is_home_or_desktop;
+	gboolean file_is_special_link;
+	gboolean can_move_file_to_trash;
+	gboolean can_delete_file;
 
 	if (event->button == 3) {
 		gboolean unmount_is_eject = FALSE;
 		gboolean show_unmount = FALSE;
-		GnomeVFSVolume *volume = NULL;
+		GMount *mount = NULL;
 		
 		if (!gtk_tree_view_get_path_at_pos (treeview, event->x, event->y,
 						    &path, NULL, NULL, NULL)) {
@@ -723,7 +697,6 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 		gtk_tree_view_get_cursor (view->details->tree_widget, &cursor_path, NULL);
 		gtk_tree_view_set_cursor (view->details->tree_widget, path, NULL, FALSE);
 		gtk_tree_path_free (path);
-		uri = nautilus_file_get_uri (view->details->popup_file);
 		
 		gtk_widget_set_sensitive (view->details->popup_open_in_new_window,
 			nautilus_file_is_directory (view->details->popup_file));
@@ -737,13 +710,30 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 							copied_files_atom,
 							clipboard_contents_received_callback, view);
 		}
-		gtk_widget_set_sensitive (view->details->popup_trash, can_move_uri_to_trash (uri));
-		g_free (uri);
+		can_move_file_to_trash = nautilus_file_can_trash (view->details->popup_file);
+		gtk_widget_set_sensitive (view->details->popup_trash, can_move_file_to_trash);
 		
-		volume = fm_tree_model_get_volume_for_root_node_file (view->details->child_model, view->details->popup_file);
-		if (volume) {
-			show_unmount = TRUE;
-			unmount_is_eject = eject_for_type (gnome_vfs_volume_get_device_type (volume));
+		if (show_delete_command_auto_value) {
+			parent_file_is_writable = is_parent_writable (view->details->popup_file);
+			file_is_home_or_desktop = nautilus_file_is_home (view->details->popup_file)
+				|| nautilus_file_is_desktop_directory (view->details->popup_file);
+			file_is_special_link = NAUTILUS_IS_DESKTOP_ICON_FILE (view->details->popup_file);
+			
+			can_delete_file = parent_file_is_writable 
+				&& !file_is_home_or_desktop
+				&& !file_is_special_link;
+
+			gtk_widget_show (view->details->popup_delete);
+			gtk_widget_set_sensitive (view->details->popup_delete, can_delete_file);
+		} else {
+			gtk_widget_hide (view->details->popup_delete);
+		}
+		
+		mount = fm_tree_model_get_mount_for_root_node_file (view->details->child_model, view->details->popup_file);
+		if (mount) {
+			show_unmount = g_mount_can_unmount (mount) || g_mount_can_eject (mount);
+			/* TODO: show both unmount and eject if there are more than one volume for the drive */
+			unmount_is_eject = g_mount_can_eject (mount);
 		} 
 		
 		gtk_label_set_text (GTK_LABEL (GTK_BIN (GTK_MENU_ITEM (view->details->popup_unmount))->child),
@@ -790,7 +780,7 @@ fm_tree_view_activate_file (FMTreeView *view,
 	view->details->activation_file = nautilus_file_ref (file);
 	view->details->activation_in_new_window = open_in_new_window;
 		
-	attributes = NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI;
+	attributes = NAUTILUS_FILE_ATTRIBUTE_INFO | NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	nautilus_file_call_when_ready (view->details->activation_file, attributes,
 				       got_activation_uri_callback, view);
 }
@@ -810,14 +800,14 @@ fm_tree_view_open_in_new_window_cb (GtkWidget *menu_item,
 }
 
 static void
-new_folder_done (const char *new_folder_uri, gpointer data)
+new_folder_done (GFile *new_folder, gpointer data)
 {
 	GList *list;
 
 	/* show the properties window for the newly created
 	 * folder so the user can change its name
 	 */
-	list = g_list_prepend (NULL, nautilus_file_get (new_folder_uri));
+	list = g_list_prepend (NULL, nautilus_file_get (new_folder));
 
 	fm_properties_window_present (list, GTK_WIDGET (data));
 
@@ -1015,25 +1005,55 @@ fm_tree_view_paste_cb (GtkWidget *menu_item,
 					paste_into_clipboard_received_callback, view);
 }
 
+static GtkWindow *
+fm_tree_view_get_containing_window (FMTreeView *view)
+{
+	GtkWidget *window;
+
+	g_assert (FM_IS_TREE_VIEW (view));
+
+	window = gtk_widget_get_ancestor (GTK_WIDGET (view), GTK_TYPE_WINDOW);
+	if (window == NULL) {
+		return NULL;
+	}
+
+	return GTK_WINDOW (window);
+}
+
 static void
 fm_tree_view_trash_cb (GtkWidget *menu_item,
 		       FMTreeView *view)
 {
 	GList *list;
-	char *directory_uri;
 
-	directory_uri = nautilus_file_get_uri (view->details->popup_file);
-	
-	if (can_move_uri_to_trash (directory_uri))
-	{
-		list = g_list_prepend (NULL, g_strdup (directory_uri));
-	
-		nautilus_file_operations_copy_move (list, NULL, 
-						    EEL_TRASH_URI, GDK_ACTION_MOVE, GTK_WIDGET (view->details->tree_widget),
-						    NULL, NULL);
+	if (!nautilus_file_can_trash (view->details->popup_file)) {
+		return;
 	}
 	
-	g_free (directory_uri);
+	list = g_list_prepend (NULL,
+			       nautilus_file_get_location (view->details->popup_file));
+	
+	nautilus_file_operations_trash_or_delete (list, 
+						  fm_tree_view_get_containing_window (view),
+						  NULL, NULL);
+	eel_g_object_list_free (list);
+}
+
+static void
+fm_tree_view_delete_cb (GtkWidget *menu_item,
+		        FMTreeView *view)
+{
+	GList *location_list;
+		
+	if (!show_delete_command_auto_value) {
+		return;
+	}
+	
+	location_list = g_list_prepend (NULL,
+					nautilus_file_get_location (view->details->popup_file));
+	
+	nautilus_file_operations_delete (location_list, fm_tree_view_get_containing_window (view), NULL, NULL);
+	eel_g_object_list_free (location_list);
 }
 
 static void
@@ -1049,44 +1069,23 @@ fm_tree_view_properties_cb (GtkWidget *menu_item,
         nautilus_file_list_free (list);
 }
 
-static void
-volume_or_drive_unmounted_callback (gboolean succeeded,
-				    char *error,
-				    char *detailed_error,
-				    gpointer data)
-{
-	gboolean eject;
-
-	eject = GPOINTER_TO_INT (data);
-	if (!succeeded) {
-		if (eject) {
-			eel_show_error_dialog_with_details (error, NULL, detailed_error, NULL);
-		} else {
-			eel_show_error_dialog_with_details (error, NULL, detailed_error, NULL);
-		}
-	}
-}
-
-
 static void
 fm_tree_view_unmount_cb (GtkWidget *menu_item,
-			    FMTreeView *view)
+			 FMTreeView *view)
 {
 	NautilusFile *file = view->details->popup_file;
-	GnomeVFSVolume *volume;
+	GMount *mount;
 	
 	if (file == NULL) {
 		return;
 	}
 
-	volume = fm_tree_model_get_volume_for_root_node_file (view->details->child_model, file);
+	mount = fm_tree_model_get_mount_for_root_node_file (view->details->child_model, file);
 	
-	if (volume != NULL) {
-		if (eject_for_type (gnome_vfs_volume_get_device_type (volume))) {
-			gnome_vfs_volume_eject (volume, volume_or_drive_unmounted_callback, GINT_TO_POINTER (TRUE));
-		} else {
-			gnome_vfs_volume_unmount (volume, volume_or_drive_unmounted_callback, GINT_TO_POINTER (FALSE));
-		}
+	if (mount != NULL) {
+		nautilus_file_operations_unmount_mount (fm_tree_view_get_containing_window (view),
+							mount,
+							g_mount_can_eject (mount), TRUE);
 	}
 }
 
@@ -1168,7 +1167,7 @@ create_popup_menu (FMTreeView *view)
 	eel_gtk_menu_append_separator (GTK_MENU (popup));
 	
 	/* add the "move to trash" menu item */
-	menu_image = gtk_image_new_from_icon_name ("user-trash",
+	menu_image = gtk_image_new_from_icon_name (NAUTILUS_ICON_TRASH,
 						   GTK_ICON_SIZE_MENU);
 	gtk_widget_show (menu_image);
 	menu_item = gtk_image_menu_item_new_with_label (_("Move to Trash"));
@@ -1181,6 +1180,15 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_trash = menu_item;
 	
+	/* add the "delete" menu item */
+	menu_item = gtk_menu_item_new_with_mnemonic (_("_Delete"));
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (fm_tree_view_delete_cb),
+			  view);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+	view->details->popup_delete = menu_item;
+	
 	eel_gtk_menu_append_separator (GTK_MENU (popup));
 
 	/* add the "Unmount" menu item */
@@ -1213,10 +1221,11 @@ create_tree (FMTreeView *view)
 {
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
-	GnomeVFSVolumeMonitor *volume_monitor;
+	GVolumeMonitor *volume_monitor;
 	char *home_uri;
-	GList *volumes, *l;
+	GList *mounts, *l;
 	char *location;
+	GIcon *icon;
 	
 	view->details->child_model = fm_tree_model_new ();
 	view->details->sort_model = GTK_TREE_MODEL_SORT
@@ -1232,27 +1241,33 @@ create_tree (FMTreeView *view)
 		(view->details->child_model, "row_loaded",
 		 G_CALLBACK (row_loaded_callback),
 		 view, G_CONNECT_AFTER);
-	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
-	fm_tree_model_add_root_uri (view->details->child_model, home_uri, _("Home Folder"), "gnome-fs-home", NULL);
+	home_uri = nautilus_get_home_directory_uri ();
+	icon = g_themed_icon_new (NAUTILUS_ICON_HOME);
+	fm_tree_model_add_root_uri (view->details->child_model, home_uri, _("Home Folder"), icon, NULL);
+	g_object_unref (icon);
 	g_free (home_uri);
-	fm_tree_model_add_root_uri (view->details->child_model, "file:///", _("File System"), "gnome-fs-directory", NULL);
-#ifdef NOT_YET_USABLE
-	fm_tree_model_add_root_uri (view->details->child_model, "network:///", _("Network Neighbourhood"), "gnome-fs-network", NULL);
+	icon = g_themed_icon_new (NAUTILUS_ICON_FILESYSTEM);
+	fm_tree_model_add_root_uri (view->details->child_model, "file:///", _("File System"), icon, NULL);
+	g_object_unref (icon);
+#ifdef NOT_YET_USABLE /* Do we really want this? */
+	icon = g_themed_icon_new (NAUTILUS_ICON_NETWORK);
+	fm_tree_model_add_root_uri (view->details->child_model, "network:///", _("Network Neighbourhood"), icon, NULL);
+	g_object_unref (icon);
 #endif
 	
-	volume_monitor = gnome_vfs_get_volume_monitor ();
-	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (volume_monitor);
-	volumes = g_list_sort (volumes, (GCompareFunc) gnome_vfs_volume_compare);
-	for (l = volumes; l != NULL; l = l->next) {
-		add_root_for_volume (view, l->data);
-		gnome_vfs_volume_unref (l->data);
+	volume_monitor = g_volume_monitor_get ();
+	view->details->volume_monitor = volume_monitor;
+	mounts = g_volume_monitor_get_mounts (volume_monitor);
+	for (l = mounts; l != NULL; l = l->next) {
+		add_root_for_mount (view, l->data);
+		g_object_unref (l->data);
 	}
-	g_list_free (volumes);
+	g_list_free (mounts);
 	
-	g_signal_connect_object (volume_monitor, "volume_mounted",
-				 G_CALLBACK (volume_mounted_callback), view, 0);
-	g_signal_connect_object (volume_monitor, "volume_unmounted",
-				 G_CALLBACK (volume_unmounted_callback), view, 0);
+	g_signal_connect_object (volume_monitor, "mount_added",
+				 G_CALLBACK (mount_added_callback), view, 0);
+	g_signal_connect_object (volume_monitor, "mount_removed",
+				 G_CALLBACK (mount_removed_callback), view, 0);
 	
 	g_object_unref (view->details->child_model);
 
@@ -1400,9 +1415,6 @@ fm_tree_view_init (FMTreeView *view)
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
 				      filtering_changed_callback, view);
 	
-	g_signal_connect_object (nautilus_icon_factory_get(), "icons_changed",
-				 G_CALLBACK (theme_changed_callback), view, 0);  
-
 	view->details->popup_file = NULL;
 	create_popup_menu (view);
 }
@@ -1453,6 +1465,8 @@ fm_tree_view_finalize (GObject *object)
 		g_free (view->details->selection_location);
 	}
 
+	g_object_unref (view->details->volume_monitor);
+	
 	g_free (view->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -1465,6 +1479,9 @@ fm_tree_view_class_init (FMTreeViewClass *class)
 	G_OBJECT_CLASS (class)->finalize = fm_tree_view_finalize;
 	
 	copied_files_atom = gdk_atom_intern ("x-special/gnome-copied-files", FALSE);
+
+	eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_ENABLE_DELETE,
+					  &show_delete_command_auto_value);	
 }
 
 static const char *
@@ -1569,4 +1586,3 @@ fm_tree_view_register (void)
 {
         nautilus_module_add_type (fm_tree_view_provider_get_type ());
 }
-
